*Size-Adapted Bond Liquidity Measures and Their Asset Pricing Implications, Journal of Financial Economics;
*July 28, 2022;
* (c) Michael Reichenbacher and Philipp Schuster
	  Karlsruhe Institute of Technology and University of Stuttgart
	  philipp.schuster@bwi.uni-stuttgart.de

*Needs the following files as input:
	- Trade data: File that contains the filtered trade data (all error correction filters 
	  should be applied). The following columns are necessary (which correspond to 
	  original column names in enhanced TRACE):
		* cusip_id (CUSIP of the bond)
		* trd_exctn_dt (execution date of the trade)
		* rptd_pr (price of the trade)
		* rpt_side_cd (buy/sell indicator, D: interdealer trade, 
		  B: dealer buys (bid), S: dealer sells (ask));

*The user should adapt the file names and the path to the folder 
 containing the data set;
%LET IN_Trace = trace_filtered;
Libname SizeAdap 'F:\SizeAdaptedLiquidity';

proc sql;
create table SizeAdap.AvgBidAsk_Sample as
select 
 cusip_id || '_' || put(year(trd_exctn_dt),4.) || '_' || put(month(trd_exctn_dt),2.) || '_' || put(day(trd_exctn_dt),2.) as 
 CusipObservationDay, 
 RPTD_PR, rpt_side_cd, trd_exctn_dt, trd_exctn_tm, entrd_vol_qt, log(entrd_vol_qt) as log_vol,
 entrd_vol_qt*RPTD_PR/100 as dollar_vol, log(entrd_vol_qt*RPTD_PR/100) as log_dollar_vol
from SizeAdap.&IN_Trace
where (rpt_side_cd = 'B' or rpt_side_cd = 'S') 
group by CusipObservationDay
having count(distinct rpt_side_cd)>=2 /*ensures that there are both bid and ask prices*/
order by cusip_id, trd_exctn_dt, rpt_side_cd;
quit;
