*Size-Adapted Bond Liquidity Measures and Their Asset Pricing Implications, Journal of Financial Economics;
*July 28, 2022;
* (c) Michael Reichenbacher and Philipp Schuster
	  Karlsruhe Institute of Technology and University of Stuttgart
	  philipp.schuster@bwi.uni-stuttgart.de


*Needs the following files as input:
	- Trade data: File that contains the filtered trade data (all error correction filters 
	  should be applied). The following columns are necessary (which correspond to 
	  original column names in enhanced TRACE):
		* cusip_id (CUSIP of the bond)
		* trd_exctn_dt (execution date of the trade)
		* rptd_pr (price of the trade)
		* rpt_side_cd (buy/sell indicator, D: interdealer trade, 
		  B: dealer buys (bid), S: dealer sells (ask))
	- File with consensus prices (e.g. from Bloomberg or Markit, in the paper, we use the closing price (px_last) from pricing source BGN in Bloomberg). The following columns are necessary:
		* quotedate
		* cusip
		* consensusPrice;

*The user should adapt the file names and the path to the folder 
 containing the data set;
%LET IN_Trace = trace_filtered;
%LET IN_Bloomberg = Bloomberg_BGN;
Libname SizeAdap 'F:\SizeAdaptedLiquidity';


proc sql;
create table SizeAdap.Schultz_Sample as
select t.cusip_id, t.cusip_id || '_' || put(year(t.trd_exctn_dt),4.) || '_' || put(month(t.trd_exctn_dt),2.) as CusipObservationMonth, 
cusip_id || '_' || put(year(t.trd_exctn_dt),4.) || '_' || put(month(t.trd_exctn_dt),2.) || '_' || put(day(t.trd_exctn_dt),2.) as 
CusipObservationDay, t.entrd_vol_qt, t.entrd_vol_qt*t.RPTD_PR/100 as dollar_vol,
t.RPTD_PR, t.rpt_side_cd, t.trd_exctn_dt, t.TRD_EXCTN_TM, 
c.consensusPrice, 
t.RPTD_PR - c.consensusPrice as Delta, (t.RPTD_PR - c.consensusPrice)/c.consensusPrice as rel_Delta,
case 
	when rpt_side_cd = 'D' then 0
	when rpt_side_cd = 'B' then -1
	when rpt_side_cd = 'S' then 1
end as dummy
from SizeAdap.&IN_Trace as t, SizeAdap.&IN_Bloomberg as c
where t.Cusip_id = c.cusip and 
	t.TRD_EXCTN_DT = c.quoteDate 
	and c.consensusPrice ne .
group by CusipObservationMonth
having count(distinct  t.rpt_side_cd)>=2 /*at least two different sides, which is the minimum requirement so that the regression is identified.*/
order by cusip_id, trd_exctn_dt, TRD_EXCTN_TM;
quit;
