/*Calculates size-adapted Schultz spread measures based on the functional forms provided for download.*/
*The user should adapt the path to the folder containing the data set with the functional form that we provide for download;

%let dir_functionalForm = F:\SizeAdaptedLiquidity\FunctionalForms;


PROC IMPORT OUT= SIZEADAP.Schultz_c_vol 
            DATAFILE= "&dir_functionalForm.\c_vol_Schultz.csv" 
            DBMS=CSV REPLACE;
     GETNAMES=YES;
     DATAROW=2; 
RUN;
PROC IMPORT OUT= SIZEADAP.Schultz_norm_fact 
            DATAFILE= "&dir_functionalForm.\normalization_factor_Schultz.csv" 
            DBMS=CSV REPLACE;
     GETNAMES=YES;
     DATAROW=2; 
RUN;


*create volume weighting scheme (see Internet Appendix C.1);
data vol_weight_seg;
input segment lower_bound upper_bound;
cards;
1 0 17500
2 17500 37500
3 37500 75000
4 75000 150000
5 150000 350000
6 350000 750000
7 750000 1500000
8 1500000 3500000
9 3500000 7500000
10 7500000 15000000
11 15000000 100000000000
run;

proc transpose data= SIZEADAP.Schultz_c_vol out=functional_form;
by entrd_vol_qt;
quit;

proc sql;
create table functional_form as 
select input(substr(_NAME_,7,4),4.) as year, input(substr(_NAME_,12,2),2.) as month, entrd_vol_qt, _NAME_, col1 as c_vol
from functional_form
order by _name_, entrd_vol_qt;
quit;

/*weight by inverse of the number of observations (see Appendix C.1)*/
proc sql;
create table groups as
select distinct substr(s.CusipObservationDay,11,7) as ObservationMonth, g.*, count(*) as nobs, 1/count(*) * 1000000 as weight_groups /*multiply with 1000000 for numerical reasons (the multiplier cancels out)*/
from vol_weight_seg g, SizeAdap.Schultz_sample s
where g.lower_bound<=s.entrd_vol_qt<g.upper_bound
group by calculated ObservationMonth, segment;
quit;

proc sql;
create table schultz_npar_input as
select s.cusipObservationDay, s.cusipObservationMonth, min(s.entrd_vol_qt,20000000) as entrd_vol_qt, s.rel_delta, s.Dummy, g.weight_groups, g.segment
from SizeAdap.Schultz_sample s, groups g
where g.lower_bound<=s.entrd_vol_qt<g.upper_bound
	and g.ObservationMonth=substr(s.CusipObservationDay,11,7)
order by cusipObservationDay, entrd_vol_qt, rel_delta, Dummy;
quit;

proc sql;
create table schultz_2nd_step_input as
select x.CusipObservationDay, x.CusipObservationMonth, x.entrd_vol_qt, x.Dummy, x.rel_delta, x.weight_groups,
	y.c_vol as tc_estimate label=''
from schultz_npar_input x, functional_form y
where x.entrd_vol_qt=y.entrd_vol_qt
	and y.year = input(substr(x.cusipObservationMonth,11,4),4.) and y.month = input(substr(x.cusipObservationMonth,16,2),2.)
order by x.cusipObservationDay, x.entrd_vol_qt, x.rel_delta, x.Dummy;
quit;

*calculate size-adapted measure;
options nonotes nosource nosource2;
proc nlin data=schultz_2nd_step_input method=marquardt Outest=reg_2nd_step noprint; 
parms c = 0	c_0=0;
	model rel_Delta = c_0 +  c*(Dummy*tc_estimate);
_weight_ =weight_groups;
bounds c >= 0;
by CusipObservationMonth;
run;
options notes source source2;

*Save final results; 
proc sql;
create table sizeAdap.Schultz_A_size_results as
select r.CusipObservationMonth, r.c * n.normalization_factor as SizeAdap_Schultz
from reg_2nd_step as r, SizeAdap.Schultz_norm_fact n
where r._TYPE_ = 'FINAL'
	and input(substr(r.cusipObservationMonth,11,4),4.)=n.year and  input(substr(r.cusipObservationMonth,16,2),2.)=n.month
order by CusipObservationMonth;
quit; 

